#!/bin/sh 

# IHS fork of WAS libsgrabber.sh + AIX support
# To use in gdb, invoke gdb -ex "set solib-search-path /tmp/unpackedlibs". If adding GSkit symbols, add them first.
# In DBX, use -p

LANG=C
LC_ALL=C

if [ $# -lt 3 ]; then
  echo "./libsGrabber.sh <path to executable> <path to core> output-dir"
  exit 1
fi

file "$1"
file "$2"

# check files are executable, core and seq
if ! test -x "$1" ; then 
  echo "$1 does not look like an executable"
  exit 2
fi

if ! file $2 | grep "core file" > /dev/null; then
  echo "$2 does not look like a core"
  exit 3
fi

OS=`uname`

case $OS in
  AIX)
    snapcore -r
    snapcore $2 $1
    cp -r /tmp/snapcore "$3/"
    ;;
  Linux)
    # Write GDB command script

    mkdir -p $3/httpdlibs
    gdb $1 $2 --eval-command "info sharedlibrary" --eval-command "quit"  > $3/grablibs.log
    # Filter shared libraries used
    grep -i "Yes" $3/grablibs.log | cut -d " " -f 6- | tr -d " " > $3/grablibs2.log

    for i in `cat $3/grablibs2.log`; do
      i=`echo $i | sed -r -e 's/[^/]*//'`
      cp $i $3/httpdlibs
    done

    # gdb uses this but it wasn't a loaded library of the crashed process
    THREAD_DB=`grep "Using host libthread_db" $3/grablibs.log | cut -d\" -f 2`
    if [ -n "$THREAD_DB" ]; then
      cp $THREAD_DB $3/httpdlibs/
    else
      cp /lib64/libthread_db* $3/httpdlibs/
    fi

    # Tar and gzip it
    rm -f $3/libs.tar.gz
    (cd $3 && tar -cf libs.tar $1 $2 httpdlibs/) 2> /dev/null
    (cd $3 && gzip libs.tar)
    rm -rf "$3/httpdlibs"
    ;;
  *)
  echo "Unsupported OS $OS"
  exit 4
esac

exit 0


